@@ -42,11 +42,6 @@ This is described in more detail in the BiNode module.
 
 ### T O D O ###
 
-# ------------- optional ----------------
-
-# TODO: maybe also allow target==EXIT_TARGET during training
-#     would have to modify _train_node_single_phase
-
 # TODO: add a target seperator that does not remove the key. Could use
 #    -> remove key
 #    --> remove one '-' on entry
@@ -70,7 +65,7 @@ This is described in more detail in the BiNode module.
 #    is in place.
 
 # TODO: Implement more internal checks for node output result?
-#    Check that last element is not None? Use assume?
+#    Check that last element is not None?
 
 # TODO: implement switchlayer, a layer where each column represents a different
 #    target, so the target value determines which nodes are used
@@ -79,22 +74,22 @@ This is described in more detail in the BiNode module.
 #    or enable some kind of folding (might be possible via CSS like suckerfish)
 
 
-from binode import (
+from .binode import (
     BiNodeException, BiNode, PreserveDimBiNode, MSG_ID_SEP, binode_coroutine
 )
-from biclassifier import BiClassifier
-from biflow import (
+from .biclassifier import BiClassifier
+from .biflow import (
     MessageResultContainer, BiFlowException, BiFlow, BiCheckpointFlow,
     EXIT_TARGET
 )
 # the inspection stuff is considered a core functionality
-from inspection import *
+from .inspection import *
 
-from test import test
+from .test import test
 
-import nodes
-import hinet
-import parallel
+from . import nodes
+from . import hinet
+from . import parallel
 
 del binode
 del biflow
