@@ -1,8 +1,8 @@
 
-from parallelbiflow import (
+from .parallelbiflow import (
     BiFlowTrainCallable, BiFlowExecuteCallable,
     ParallelBiFlowException, ParallelBiFlow, ParallelCheckpointBiFlow)
-from parallelbihinet import ParallelCloneBiLayer
+from .parallelbihinet import ParallelCloneBiLayer
 
 del parallelbiflow
 del parallelbihinet
