@@ -1,4 +1,4 @@
-from caching_extension import (activate_caching, deactivate_caching,
+from .caching_extension import (activate_caching, deactivate_caching,
                                cache, set_cachedir,
                                __doc__, __docformat__)
 
