@@ -1,29 +1,36 @@
+from future.utils import with_metaclass
+import io as StringIO
+from .repo_revision import get_git_revision
+import mdp
+# python 2/3 compatibility
+try:
+    from inspect import getfullargspec as getargs
+except ImportError:
+    from inspect import getargspec as getargs
+import tempfile
+import os
+import sys
+from builtins import object
+from builtins import str
+from builtins import zip
 from future import standard_library
 standard_library.install_aliases()
-from builtins import zip
-from builtins import str
-from builtins import object
-import sys
-import os
-import tempfile
-import inspect
-import mdp
-from .repo_revision import get_git_revision
-import io as StringIO
-from future.utils import with_metaclass
 
 
 __docformat__ = "restructuredtext en"
 
+
 class MetaConfig(type):
     """Meta class for config object to allow for pretty printing
     of class config (as we never instantiate it)"""
+
     def __str__(self):
         return self.info()
 
     def __repr__(self):
         return self.info()
 
+
 class config(with_metaclass(MetaConfig, object)):
     """Provide information about optional dependencies.
 
@@ -81,7 +88,7 @@ class config(with_metaclass(MetaConfig, object)):
         def __init__(self, name, version=None, failmsg=None):
             assert (version is not None) + (failmsg is not None) == 1
 
-            self.version = str(version) # convert e.g. exception to str
+            self.version = str(version)  # convert e.g. exception to str
             self.failmsg = str(failmsg) if failmsg is not None else None
 
             global config
@@ -150,7 +157,7 @@ class config(with_metaclass(MetaConfig, object)):
                     numx: scipy 0.9.0
                   symeig: scipy.linalg.eigh
 
-        This function is used to provide the py.test report header and
+        This function is used to provide the pytest report header and
         footer.
         """
         listable_features = [(f[4:].replace('_', ' '), getattr(cls, f))
@@ -160,6 +167,98 @@ class config(with_metaclass(MetaConfig, object)):
         return '\n'.join('%*s: %r' % (maxlen+1, f[0], f[1])
                          for f in listable_features)
 
+
+# In scipy >= 1.4.0 several deprecated members cause issues, see
+# https://github.com/mdp-toolkit/mdp-toolkit/issues/69
+# https://scipy.github.io/devdocs/release.1.4.0.html#scipy-deprecations
+# We replace the following members by their numpy counterparts in
+# case of scipy >= 1.4.0. It lists all common callable members of
+# numpy 1.17.4 and scipy 1.4.0.
+scipy_1_4_depr = (
+'AxisError', 'ComplexWarning', 'DataSource', 'MachAr',
+'ModuleDeprecationWarning', 'RankWarning', 'TooHardError',
+'VisibleDeprecationWarning', '_add_newdoc_ufunc', 'absolute', 'add',
+'add_docstring', 'add_newdoc', 'add_newdoc_ufunc', 'alen', 'all', 'allclose',
+'alltrue', 'amax', 'amin', 'angle', 'any', 'append', 'apply_along_axis',
+'apply_over_axes', 'arange', 'arccos', 'arccosh', 'arcsin', 'arcsinh',
+'arctan', 'arctan2', 'arctanh', 'argmax', 'argmin', 'argpartition',
+'argsort', 'argwhere', 'around', 'array', 'array2string', 'array_equal',
+'array_equiv', 'array_repr', 'array_split', 'array_str', 'asanyarray',
+'asarray', 'asarray_chkfinite', 'ascontiguousarray', 'asfarray',
+'asfortranarray', 'asmatrix', 'asscalar', 'atleast_1d', 'atleast_2d',
+'atleast_3d', 'average', 'bartlett', 'base_repr', 'binary_repr', 'bincount',
+'bitwise_and', 'bitwise_not', 'bitwise_or', 'bitwise_xor', 'blackman',
+'block', 'bmat', 'bool8', 'bool_', 'broadcast', 'broadcast_arrays',
+'broadcast_to', 'busday_count', 'busday_offset', 'busdaycalendar', 'byte',
+'byte_bounds', 'bytes0', 'bytes_', 'can_cast', 'cbrt', 'cdouble', 'ceil',
+'cfloat', 'character', 'chararray', 'choose', 'clip', 'clongdouble',
+'clongfloat', 'column_stack', 'common_type', 'compare_chararrays',
+'complex128', 'complex256', 'complex64', 'complex_', 'complexfloating',
+'compress', 'concatenate', 'conj', 'conjugate', 'convolve', 'copy', 'copysign',
+'copyto', 'corrcoef', 'correlate', 'cos', 'cosh', 'count_nonzero', 'cov',
+'cross', 'csingle', 'cumprod', 'cumproduct', 'cumsum', 'datetime64',
+'datetime_as_string', 'datetime_data', 'deg2rad', 'degrees', 'delete',
+'deprecate', 'deprecate_with_doc', 'diag', 'diag_indices', 'diag_indices_from',
+'diagflat', 'diagonal', 'diff', 'digitize', 'disp', 'divide', 'divmod', 'dot',
+'double', 'dsplit', 'dstack', 'dtype', 'ediff1d', 'einsum', 'einsum_path',
+'empty', 'empty_like', 'equal', 'errstate', 'exp', 'exp2', 'expand_dims',
+'expm1', 'extract', 'eye', 'fabs', 'fastCopyAndTranspose', 'fill_diagonal',
+'find_common_type', 'finfo', 'fix', 'flatiter', 'flatnonzero', 'flexible',
+'flip', 'fliplr', 'flipud', 'float128', 'float16', 'float32', 'float64',
+'float_', 'float_power', 'floating', 'floor', 'floor_divide', 'fmax', 'fmin',
+'fmod', 'format_float_positional', 'format_float_scientific', 'format_parser',
+'frexp', 'frombuffer', 'fromfile', 'fromfunction', 'fromiter', 'frompyfunc',
+'fromregex', 'fromstring', 'full', 'full_like', 'fv', 'gcd', 'generic',
+'genfromtxt', 'geomspace', 'get_array_wrap', 'get_include', 'get_printoptions',
+'getbufsize', 'geterr', 'geterrcall', 'geterrobj', 'gradient', 'greater',
+'greater_equal', 'half', 'hamming', 'hanning', 'heaviside', 'histogram',
+'histogram2d', 'histogram_bin_edges', 'histogramdd', 'hsplit', 'hstack',
+'hypot', 'i0', 'identity', 'iinfo', 'imag', 'in1d', 'indices', 'inexact',
+'info', 'inner', 'insert', 'int0', 'int16', 'int32', 'int64', 'int8', 'int_',
+'int_asbuffer', 'intc', 'integer', 'interp', 'intersect1d', 'intp', 'invert',
+'ipmt', 'irr', 'is_busday', 'isclose', 'iscomplex', 'iscomplexobj', 'isfinite',
+'isfortran', 'isin', 'isinf', 'isnan', 'isnat', 'isneginf', 'isposinf',
+'isreal', 'isrealobj', 'isscalar', 'issctype', 'issubclass_', 'issubdtype',
+'issubsctype', 'iterable', 'ix_', 'kaiser', 'kron', 'lcm', 'ldexp',
+'left_shift', 'less', 'less_equal', 'lexsort', 'linspace', 'load', 'loads',
+'loadtxt', 'log', 'log10', 'log1p', 'log2', 'logaddexp', 'logaddexp2',
+'logical_and', 'logical_not', 'logical_or', 'logical_xor', 'logspace', 'long',
+'longcomplex', 'longdouble', 'longfloat', 'longlong', 'lookfor', 'mafromtxt',
+'mask_indices', 'mat', 'matmul', 'matrix', 'maximum', 'maximum_sctype',
+'may_share_memory', 'mean', 'median', 'memmap', 'meshgrid', 'min_scalar_type',
+'minimum', 'mintypecode', 'mirr', 'mod', 'modf', 'moveaxis', 'msort',
+'multiply', 'nan_to_num', 'nanargmax', 'nanargmin', 'nancumprod', 'nancumsum',
+'nanmax', 'nanmean', 'nanmedian', 'nanmin', 'nanpercentile', 'nanprod',
+'nanquantile', 'nanstd', 'nansum', 'nanvar', 'ndarray', 'ndenumerate',
+'ndfromtxt', 'ndim', 'ndindex', 'nditer', 'negative', 'nested_iters',
+'nextafter', 'nonzero', 'not_equal', 'nper', 'npv', 'number', 'obj2sctype',
+'object0', 'object_', 'ones', 'ones_like', 'outer', 'packbits', 'pad',
+'partition', 'percentile', 'piecewise', 'place', 'pmt', 'poly', 'poly1d',
+'polyadd', 'polyder', 'polydiv', 'polyfit', 'polyint', 'polymul', 'polysub',
+'polyval', 'positive', 'power', 'ppmt', 'printoptions', 'prod', 'product',
+'promote_types', 'ptp', 'put', 'put_along_axis', 'putmask', 'pv', 'quantile',
+'rad2deg', 'radians', 'rank', 'rate', 'ravel', 'ravel_multi_index', 'real',
+'real_if_close', 'recarray', 'recfromcsv', 'recfromtxt', 'reciprocal',
+'record', 'remainder', 'repeat', 'require', 'reshape', 'resize', 'result_type',
+'right_shift', 'rint', 'roll', 'rollaxis', 'roots', 'rot90', 'round_',
+'row_stack', 'safe_eval', 'save', 'savetxt', 'savez', 'savez_compressed',
+'sctype2char', 'searchsorted', 'select', 'set_numeric_ops', 'set_printoptions',
+'set_string_function', 'setbufsize', 'setdiff1d', 'seterr', 'seterrcall',
+'seterrobj', 'setxor1d', 'shape', 'shares_memory', 'short', 'show_config',
+'sign', 'signbit', 'signedinteger', 'sin', 'sinc', 'single', 'singlecomplex',
+'sinh', 'size', 'sometrue', 'sort', 'sort_complex', 'source', 'spacing',
+'split', 'sqrt', 'square', 'squeeze', 'stack', 'std', 'str0', 'str_',
+'string_', 'subtract', 'sum', 'swapaxes', 'take', 'take_along_axis', 'tan',
+'tanh', 'tensordot', 'test', 'tile', 'timedelta64', 'trace', 'transpose',
+'trapz', 'tri', 'tril', 'tril_indices', 'tril_indices_from', 'trim_zeros',
+'triu', 'triu_indices', 'triu_indices_from', 'true_divide', 'trunc',
+'typename', 'ubyte', 'ufunc', 'uint', 'uint0', 'uint16', 'uint32', 'uint64',
+'uint8', 'uintc', 'uintp', 'ulonglong', 'unicode', 'unicode_', 'union1d',
+'unique', 'unpackbits', 'unravel_index', 'unsignedinteger', 'unwrap',
+'ushort', 'vander', 'var', 'vdot', 'vectorize', 'void', 'void0', 'vsplit',
+'vstack', 'where', 'who', 'zeros', 'zeros_like'
+)
+
 def get_numx():
     # find out the numerical extension
     # To force MDP to use one specific extension module
@@ -186,6 +285,19 @@ def get_numx():
                                version as numx_version)
             numx_description = 'scipy'
             config.ExternalDepFound('numx', 'scipy ' + numx_version.version)
+
+            # In scipy >= 1.4.0 several deprecated members cause issues, see
+            # https://github.com/mdp-toolkit/mdp-toolkit/issues/69
+            # https://scipy.github.io/devdocs/release.1.4.0.html#scipy-deprecations
+            # We have to replace them by numpy counterparts.
+            nvs = numx_version.short_version.split('.')
+            if int(nvs[0]) > 1 or int(nvs[0]) == 1 and int(nvs[1]) >= 4:
+                import numpy as np
+                for x in scipy_1_4_depr:
+                    try:
+                        setattr(numx, x, getattr(np, x))
+                    except AttributeError:
+                        pass
         except ImportError as exc:
             if USR_LABEL:
                 raise ImportError(exc)
@@ -210,19 +322,20 @@ def get_numx():
     # the test is for numx_description, not numx, because numx could
     # be imported successfully, but e.g. numx_rand could later fail.
     if numx_description is None:
-        msg = ([ "Could not import any of the numeric backends.",
-                 "Import errors:" ] +
-               [ lab+': '+str(exc) for lab, exc in list(numx_exception.items()) ]
+        msg = (["Could not import any of the numeric backends.",
+                "Import errors:"] +
+               [lab+': '+str(exc) for lab, exc in list(numx_exception.items())]
                + ["sys.path: " + str(sys.path)])
         raise ImportError('\n'.join(msg))
 
     return (numx_description, numx, numx_linalg,
             numx_fft, numx_rand, numx_version)
 
+
 def get_symeig(numx_linalg):
     # if we have scipy, check if the version of
     # scipy.linalg.eigh supports the rich interface
-    args = inspect.getargspec(numx_linalg.eigh)[0]
+    args = getargs(numx_linalg.eigh)[0]
     if len(args) > 4:
         # if yes, just wrap it
         from .utils._symeig import wrap_eigh as symeig
@@ -234,6 +347,7 @@ def get_symeig(numx_linalg):
         config.ExternalDepFound('symeig', 'symeig_fake')
     return symeig
 
+
 def _version_too_old(version, known_good):
     """Return True iff a version is smaller than a tuple of integers.
 
@@ -253,7 +367,7 @@ def _version_too_old(version, known_good):
     False
     >>> _version_too_old('0.4.devel', (0,4,3))
     """
-    for part,expected in zip(version.split('.'), known_good):
+    for part, expected in zip(version.split('.'), known_good):
         try:
             p = int(part)
         except ValueError:
@@ -264,15 +378,19 @@ def _version_too_old(version, known_good):
             break
     return False
 
+
 class _sys_stdout_replaced(object):
     "Replace systdout temporarily"
+
     def __enter__(self):
         self.sysstdout = sys.stdout
         sys.stdout = StringIO.StringIO()
         return sys.stdout
+
     def __exit__(self, *args):
         sys.stdout = self.sysstdout
 
+
 def _pp_needs_monkeypatching():
     # only run this function the first time mdp is imported
     # otherwise reload(mdp) breaks
@@ -309,6 +427,7 @@ def _pp_needs_monkeypatching():
 
     return mdp._pp_needs_monkeypatching
 
+
 def set_configuration():
     # set python version
     config.ExternalDepFound('python', '.'.join([str(x)
@@ -386,17 +505,14 @@ def set_configuration():
 
     # libsvm
     try:
-        import svm as libsvm
-        libsvm.libsvm
+        import libsvm
     except ImportError as exc:
         config.ExternalDepFailed('libsvm', exc)
-    except AttributeError as exc:
-        config.ExternalDepFailed('libsvm', 'libsvm version >= 2.91 required')
     else:
         if os.getenv('MDP_DISABLE_LIBSVM'):
             config.ExternalDepFailed('libsvm', 'disabled')
         else:
-            config.ExternalDepFound('libsvm', libsvm.libsvm._name)
+            config.ExternalDepFound('libsvm', libsvm.__version__)
 
     # joblib
     try:
@@ -407,7 +523,7 @@ def set_configuration():
         version = joblib.__version__
         if os.getenv('MDP_DISABLE_JOBLIB'):
             config.ExternalDepFailed('joblib', 'disabled')
-        elif _version_too_old(version, (0,4,3)):
+        elif _version_too_old(version, (0, 4, 3)):
             config.ExternalDepFailed('joblib',
                                      'version %s is too old' % version)
         else:
@@ -427,7 +543,7 @@ def set_configuration():
     else:
         if os.getenv('MDP_DISABLE_SKLEARN'):
             config.ExternalDepFailed('sklearn', 'disabled')
-        elif _version_too_old(version, (0,6)):
+        elif _version_too_old(version, (0, 6)):
             config.ExternalDepFailed('sklearn',
                                      'version %s is too old' % version)
         else:
