@@ -20,8 +20,12 @@ still possible to define the extension nodes as classes derived from nodes.
 This keeps the code readable and is compatible with automatic code checkers
 (like the background pylint checks in the Eclipse IDE with PyDev).
 """
+from __future__ import print_function
+from builtins import str
+from builtins import object
 
 from mdp import MDPException, NodeMetaclass
+from future.utils import with_metaclass
 
 # TODO: Register the node instances as well?
 #    This would allow instance initialization when an extension is activated.
@@ -35,15 +39,24 @@ from mdp import MDPException, NodeMetaclass
 
 # name prefix used for the original attributes when they are shadowed
 ORIGINAL_ATTR_PREFIX = "_non_extension_"
-# prefix used to store the current extension name for an attribute
-EXTENSION_ATTR_PREFIX = "_extension_for_"
+# prefix used to store the current extension name for an attribute,
+# the value stored in this attribute is the extension name
+_EXTENSION_ATTR_PREFIX = "_extension_for_"
 # list of attribute names that are not affected by extensions,
-NON_EXTENSION_ATTRIBUTES = ["__module__", "__doc__", "extension_name"]
+_NON_EXTENSION_ATTRIBUTES = ["__module__", "__doc__", "extension_name"]
+
+# keys under which the global activation and deactivation functions
+# for extensions can be stored in the extension registry
+_SETUP_FUNC_ATTR = "_extension_setup"
+_TEARDOWN_FUNC_ATTR = "_extension_teardown"
 
 # dict of dicts of dicts, contains a key for each extension,
 # the inner dict maps the node types to their extension node,
 # the innermost dict then maps attribute names to values
 # (e.g. a method name to the actual function)
+#
+# For each extension there are also the special _SETUP_FUNC_ATTR and
+# _TEARDOWN_FUNC_ATTR keys.
 _extensions = dict()
 # set containing the names of the currently activated extensions
 _active_extensions = set()
@@ -62,13 +75,12 @@ def _register_attribute(ext_name, node_cls, attr_name, attr_value):
     """
     _extensions[ext_name][node_cls][attr_name] = attr_value
 
-def extension_method(ext_name, node_cls, method_name=None):
-    """Returns a function to register a function as extension method.
 
-    This function is intended to be used with the decorator syntax.
+def extension_method(extension_name, node_cls, method_name=None):
+    """Returns a decorator to register a function as an extension method.
 
     :Parameters:
-      ext_name
+      extension_name
         String with the name of the extension.
       node_cls
         Node class for which the method should be registered.
@@ -85,18 +97,62 @@ def extension_method(ext_name, node_cls, method_name=None):
         _method_name = method_name
         if not _method_name:
             _method_name = func.__name__
-        if not ext_name in _extensions:
-            err = ("No ExtensionNode base class has been defined for this "
-                   "extension.")
-            raise ExtensionException(err)
-        if not node_cls in _extensions[ext_name]:
+        if not extension_name in _extensions:
+            # creation of a new extension, add entry in dict
+            _extensions[extension_name] = dict()
+        if not node_cls in _extensions[extension_name]:
             # register this node
-            _extensions[ext_name][node_cls] = dict()
-        _register_attribute(ext_name, node_cls, _method_name, func)
+            _extensions[extension_name][node_cls] = dict()
+        _register_attribute(extension_name, node_cls, _method_name, func)
         return func
     return register_function
 
 
+def extension_setup(extension_name):
+    """Returns a decorator to register a setup function for an extension.
+
+    :Parameters:
+      extension_name
+        String with the name of the extension.
+
+    The decorated function will be called when the extension is activated.
+    
+    Note that there is also the extension_teardown decorator, which should
+    probably defined as well if there is a setup procedure.
+    """
+    def register_setup_function(func):
+        if not extension_name in _extensions:
+            # creation of a new extension, add entry in dict
+            _extensions[extension_name] = dict()
+        if _SETUP_FUNC_ATTR in _extensions[extension_name]:
+            err = "There is already a setup function for this extension."
+            raise ExtensionException(err)
+        _extensions[extension_name][_SETUP_FUNC_ATTR] = func
+        return func
+    return register_setup_function
+
+
+def extension_teardown(extension_name):
+    """Returns a decorator to register a teardown function for an extension.
+
+    :Parameters:
+      extension_name
+        String with the name of the extension.
+
+    The decorated function will be called when the extension is deactivated.
+    """
+    def register_teardown_function(func):
+        if not extension_name in _extensions:
+            # creation of a new extension, add entry in dict
+            _extensions[extension_name] = dict()
+        if _TEARDOWN_FUNC_ATTR in _extensions[extension_name]:
+            err = "There is already a teardown function for this extension."
+            raise ExtensionException(err)
+        _extensions[extension_name][_TEARDOWN_FUNC_ATTR] = func
+        return func
+    return register_teardown_function
+
+
 class ExtensionNodeMetaclass(NodeMetaclass):
     """This is the metaclass for node extension superclasses.
 
@@ -161,8 +217,8 @@ class ExtensionNodeMetaclass(NodeMetaclass):
             # make sure we only inject methods in classes which have
             # ExtensionNode as superclass
             if extension_subtree and ExtensionNode in base.__mro__:
-                for attr_name, attr_value in base.__dict__.items():
-                    if attr_name not in NON_EXTENSION_ATTRIBUTES:
+                for attr_name, attr_value in list(base.__dict__.items()):
+                    if attr_name not in _NON_EXTENSION_ATTRIBUTES:
                         # check if this attribute has not already been
                         # extended in one of the base classes
                         already_active = False
@@ -180,7 +236,7 @@ class ExtensionNodeMetaclass(NodeMetaclass):
         return ext_node_cls
 
 
-class ExtensionNode(object):
+class ExtensionNode(with_metaclass(ExtensionNodeMetaclass, object)):
     """Base class for extensions nodes.
 
     A new extension node class should override the _extension_name.
@@ -212,7 +268,6 @@ class ExtensionNode(object):
     simply prefix the method name with '_non_extension_' (this is the value
     of the `ORIGINAL_ATTR_PREFIX` constant in this module).
     """
-    __metaclass__ = ExtensionNodeMetaclass
     # override this name in a concrete extension node base class
     extension_name = None
 
@@ -234,22 +289,26 @@ def get_active_extensions():
 
 def activate_extension(extension_name, verbose=False):
     """Activate the extension by injecting the extension methods."""
-    if extension_name not in _extensions.keys():
-        err = "Unknown extension name: %s" + str(extension_name)
+    if extension_name not in list(_extensions.keys()):
+        err = "Unknown extension name: %s"%str(extension_name)
         raise ExtensionException(err)
     if extension_name in _active_extensions:
         if verbose:
-            print 'Extension %s is already active!' % extension_name
+            print('Extension %s is already active!' % extension_name)
         return
     _active_extensions.add(extension_name)
     try:
-        for node_cls, attributes in _extensions[extension_name].items():
-            for attr_name, attr_value in attributes.items():
+        if _SETUP_FUNC_ATTR in _extensions[extension_name]:
+            _extensions[extension_name][_SETUP_FUNC_ATTR]()
+        for node_cls, attributes in list(_extensions[extension_name].items()):
+            if node_cls == _SETUP_FUNC_ATTR or node_cls == _TEARDOWN_FUNC_ATTR:
+                continue
+            for attr_name, attr_value in list(attributes.items()):
                 if verbose:
                     print ("extension %s: adding %s to %s" %
                            (extension_name, attr_name, node_cls.__name__))
                 ## store the original attribute / make it available
-                ext_attr_name = EXTENSION_ATTR_PREFIX + attr_name
+                ext_attr_name = _EXTENSION_ATTR_PREFIX + attr_name
                 if attr_name in dir(node_cls):
                     if ext_attr_name in node_cls.__dict__:
                         # two extensions override the same attribute
@@ -272,20 +331,22 @@ def activate_extension(extension_name, verbose=False):
                 # store to which extension this attribute belongs, this is also
                 # used as a flag that this is an extension attribute
                 setattr(node_cls, ext_attr_name, extension_name)
-    except:
+    except Exception:
         # make sure that an incomplete activation is reverted
         deactivate_extension(extension_name)
         raise
 
 def deactivate_extension(extension_name, verbose=False):
     """Deactivate the extension by removing the injected methods."""
-    if extension_name not in _extensions.keys():
+    if extension_name not in list(_extensions.keys()):
         err = "Unknown extension name: " + str(extension_name)
         raise ExtensionException(err)
     if extension_name not in _active_extensions:
         return
-    for node_cls, attributes in _extensions[extension_name].items():
-        for attr_name in attributes.keys():
+    for node_cls, attributes in list(_extensions[extension_name].items()):
+        if node_cls == _SETUP_FUNC_ATTR or node_cls == _TEARDOWN_FUNC_ATTR:
+            continue
+        for attr_name in list(attributes.keys()):
             original_name = ORIGINAL_ATTR_PREFIX + attr_name
             if verbose:
                 print ("extension %s: removing %s from %s" %
@@ -302,8 +363,8 @@ def deactivate_extension(extension_name, verbose=False):
                 # method, otherwise we would inject unwanted methods.
                 # Note: '==' tests identity for .__func__ and .__self__,
                 #    but .im_class does not matter in Python 2.6.
-                if all(map(lambda x:getattr(x, attr_name, None) !=
-                           original_attr, node_cls.__mro__[1:])):
+                if all([getattr(x, attr_name, None) !=
+                           original_attr for x in node_cls.__mro__[1:]]):
                     setattr(node_cls, attr_name, original_attr)
                 delattr(node_cls, original_name)
             else:
@@ -315,9 +376,11 @@ def deactivate_extension(extension_name, verbose=False):
                     pass
             try:
                 # might be missing if the activation failed
-                delattr(node_cls, EXTENSION_ATTR_PREFIX + attr_name)
+                delattr(node_cls, _EXTENSION_ATTR_PREFIX + attr_name)
             except AttributeError:
                 pass
+    if _TEARDOWN_FUNC_ATTR in _extensions[extension_name]:
+        _extensions[extension_name][_TEARDOWN_FUNC_ATTR]()
     _active_extensions.remove(extension_name)
 
 def activate_extensions(extension_names, verbose=False):
@@ -403,7 +466,7 @@ class extension(object):
     """
 
     def __init__(self, ext_names):
-        if isinstance(ext_names, str):
+        if isinstance(ext_names, __builtins__['str']):
             ext_names = [ext_names]
         self.ext_names = ext_names
         self.deactivate_exts = []
