@@ -28,7 +28,9 @@ then feed the output into a Layer and each Node will get the correct input.
 """
 
 from .flownode import FlowNode
+from .flownode_online import (OnlineFlowNode, CircularOnlineFlowNode)
 from .layer import Layer, SameInputLayer, CloneLayer
+from .layer_online import OnlineLayer, SameInputOnlineLayer, CloneOnlineLayer
 from .switchboard import (
     Switchboard, SwitchboardException, MeanInverseSwitchboard,
     ChannelSwitchboard,
@@ -45,7 +47,8 @@ from .switchboard_factory import (
     FactoryDoubleRhomb2dSwitchboard
 )
 
-__all__ = ['FlowNode', 'Layer', 'SameInputLayer', 'CloneLayer',
+__all__ = ['FlowNode', 'OnlineFlowNode', 'CircularOnlineFlowNode', 'Layer', 'SameInputLayer', 'CloneLayer',
+           'OnlineLayer', 'SameInputOnlineLayer', 'CloneOnlineLayer',
            'Switchboard', 'SwitchboardException', 'ChannelSwitchboard',
            'Rectangular2dSwitchboard', 'Rectangular2dSwitchboardException',
            'DoubleRect2dSwitchboard', 'DoubleRect2dSwitchboardException',
@@ -57,7 +60,9 @@ __all__ = ['FlowNode', 'Layer', 'SameInputLayer', 'CloneLayer',
 from mdp.utils import fixup_namespace
 fixup_namespace(__name__, __all__,
                 ('flownode',
+                 'flownode_online',
                  'layer',
+                 'layer_online',
                  'switchboard',
                  'hinet_Visitor',
                  'switchboard_factory',
