@@ -1,42 +1,51 @@
 # -*- coding:utf-8 -*-
 __docformat__ = "restructuredtext en"
 
+from mdp import utils
 from .pca_nodes import WhiteningNode, PCANode
-from .sfa_nodes import SFANode, SFA2Node
+from .sfa_nodes import SFANode, SFA2Node, VartimeSFANode
 from .ica_nodes import ICANode, CuBICANode, FastICANode, TDSEPNode
 from .neural_gas_nodes import GrowingNeuralGasNode, NeuralGasNode
 from .expansion_nodes import (QuadraticExpansionNode, PolynomialExpansionNode,
-                             RBFExpansionNode, GrowingNeuralGasExpansionNode,
-                             GeneralExpansionNode)
+                              RBFExpansionNode, GrowingNeuralGasExpansionNode,
+                              GeneralExpansionNode)
 from .fda_nodes import FDANode
 from .em_nodes import FANode
 from .misc_nodes import (IdentityNode, HitParadeNode, TimeFramesNode,
-                        TimeDelayNode, TimeDelaySlidingWindowNode,
-                        EtaComputerNode, NoiseNode, NormalNoiseNode,
-                        CutoffNode, HistogramNode, AdaptiveCutoffNode)
+                         TimeDelayNode, TimeDelaySlidingWindowNode,
+                         EtaComputerNode, NoiseNode, NormalNoiseNode,
+                         CutoffNode, HistogramNode, AdaptiveCutoffNode)
 from .isfa_nodes import ISFANode
 from .rbm_nodes import RBMNode, RBMWithLabelsNode
 from .regression_nodes import LinearRegressionNode
 from .classifier_nodes import (SignumClassifier, PerceptronClassifier,
-                              SimpleMarkovClassifier,
-                              DiscreteHopfieldClassifier,
-                              KMeansClassifier, GaussianClassifier,
-                              NearestMeanClassifier, KNNClassifier)
+                               SimpleMarkovClassifier,
+                               DiscreteHopfieldClassifier,
+                               KMeansClassifier, GaussianClassifier,
+                               NearestMeanClassifier, KNNClassifier)
 from .jade import JADENode
 from .nipals import NIPALSNode
 from .lle_nodes import LLENode, HLLENode
 from .xsfa_nodes import XSFANode, NormalizeNode
+from .gsfa_nodes import GSFANode, iGSFANode
+from .recursive_expansion_nodes import (RecursiveExpansionNode,
+                                        NormalizingRecursiveExpansionNode)
 
 # import internals for use in test_suites
 from .misc_nodes import OneDimensionalHitParade as _OneDimensionalHitParade
 from .expansion_nodes import expanded_dim as _expanded_dim
 
+from .mca_nodes_online import MCANode
+from .pca_nodes_online import CCIPCANode, CCIPCAWhiteningNode
+from .sfa_nodes_online import IncSFANode
+from .stats_nodes_online import OnlineCenteringNode, OnlineTimeDiffNode
+
 __all__ = ['PCANode', 'WhiteningNode', 'NIPALSNode', 'FastICANode',
-           'CuBICANode', 'TDSEPNode', 'JADENode', 'SFANode', 'SFA2Node',
-           'ISFANode', 'XSFANode', 'FDANode', 'FANode', 'RBMNode',
+           'CuBICANode', 'TDSEPNode', 'JADENode', 'SFANode', 'SFA2Node', 'VartimeSFANode',
+           'ISFANode', 'XSFANode', 'GSFANode', 'iGSFANode', 'FDANode', 'FANode', 'RBMNode',
            'RBMWithLabelsNode', 'GrowingNeuralGasNode', 'LLENode', 'HLLENode',
            'LinearRegressionNode', 'QuadraticExpansionNode',
-           'PolynomialExpansionNode', 'RBFExpansionNode','GeneralExpansionNode',
+           'PolynomialExpansionNode', 'RBFExpansionNode', 'GeneralExpansionNode',
            'GrowingNeuralGasExpansionNode', 'NeuralGasNode', '_expanded_dim',
            'SignumClassifier',
            'PerceptronClassifier', 'SimpleMarkovClassifier',
@@ -45,7 +54,9 @@ __all__ = ['PCANode', 'WhiteningNode', 'NIPALSNode', 'FastICANode',
            'EtaComputerNode', 'HitParadeNode', 'NoiseNode', 'NormalNoiseNode',
            'TimeFramesNode', 'TimeDelayNode', 'TimeDelaySlidingWindowNode',
            'CutoffNode', 'AdaptiveCutoffNode', 'HistogramNode',
-           'IdentityNode', '_OneDimensionalHitParade']
+           'IdentityNode', '_OneDimensionalHitParade',
+           'OnlineCenteringNode', 'OnlineTimeDiffNode', 'CCIPCANode', 'CCIPCAWhiteningNode', 'MCANode',
+           'IncSFANode', 'RecursiveExpansionNode', 'NormalizingRecursiveExpansionNode', ]
 
 # nodes with external dependencies
 from mdp import config, numx_description, MDPException
@@ -70,7 +81,6 @@ if config.has_sklearn:
             __all__.append(name)
         del name
 
-from mdp import utils
 utils.fixup_namespace(__name__, __all__ + ['ICANode'],
                       ('pca_nodes',
                        'sfa_nodes',
@@ -88,6 +98,7 @@ utils.fixup_namespace(__name__, __all__ + ['ICANode'],
                        'nipals',
                        'lle_nodes',
                        'xsfa_nodes',
+                       'gsfa_nodes',
                        'convolution_nodes',
                        'shogun_svm_classifier',
                        'svm_classifiers',
@@ -98,4 +109,9 @@ utils.fixup_namespace(__name__, __all__ + ['ICANode'],
                        'scikits_nodes',
                        'numx_description',
                        'config',
+                       'stats_nodes_online',
+                       'pca_nodes_online',
+                       'mca_nodes_online',
+                       'sfa_nodes_online',
+                       'recursive_expansion_nodes',
                        ))
