@@ -1,6 +1,5 @@
 # -*- coding:utf-8; -*-
 """Wraps the algorithms defined in scikits.learn in MDP Nodes.
-
 This module is based on the 0.6.X branch of scikits.learn .
 """
 from __future__ import print_function
@@ -89,11 +88,9 @@ _HEADINGS = set(['Parameters', 'Attributes', 'Methods', 'Examples',
 
 _DOC_TEMPLATE = """
 %s
-
 This node has been automatically generated by wrapping the ``%s.%s`` class
 from the ``sklearn`` library.  The wrapped instance can be accessed
 through the ``scikits_alg`` attribute.
-
 %s
 """
 
@@ -158,14 +155,14 @@ def _gen_docstring(object, docsource=None):
 def apply_to_scikits_algorithms(current_module, action,
                                 processed_modules=None,
                                 processed_classes=None):
-    """ Function that traverses a module to find scikits algorithms.
-
+    """Function that traverses a module to find scikits algorithms.
     'sklearn' algorithms are identified by the 'fit' 'predict',
     or 'transform' methods. The 'action' function is applied to each found
     algorithm.
     
-    action -- a function that is called with as action(class_), where
-              class_ is a class that defines the 'fit' or 'predict' method
+    :param action: A function that is called with as action(class_), where
+        'class_' is a class that defines the 'fit' or 'predict' method.
+    :type action: function
     """
 
     # only consider modules and classes once
@@ -199,17 +196,17 @@ def apply_to_scikits_algorithms(current_module, action,
 
 
 _OUTPUTDIM_ERROR = """'output_dim' keyword not supported.
-
 Please set the output dimensionality using sklearn keyword
 arguments (e.g., 'n_components', or 'k'). See the docstring of this
 class for details."""
 
 def wrap_scikits_classifier(scikits_class):
     """Wrap a sklearn classifier as an MDP Node subclass.
-
     The wrapper maps these MDP methods to their sklearn equivalents:
+
     - _stop_training -> fit
     - _label -> predict
+
     """
 
     newaxis = mdp.numx.newaxis
@@ -219,7 +216,21 @@ def wrap_scikits_classifier(scikits_class):
 
         def __init__(self, input_dim=None, output_dim=None, dtype=None,
                      **kwargs):
+            """
+            Initializes an object of type 'ScikitsNode'.
 
+            :param input_dim: Dimensionality of the input.
+                Default is None.
+            :type input_dim: int
+        
+            :param output_dim: Dimensionality of the output.
+                Default is None.
+            :type output_dim: int
+        
+            :param dtype: Datatype of the input.
+                Default is None.
+            :type dtype: numpy.dtype or str
+            """
             if output_dim is not None:
                 # output_dim and n_components cannot be defined at the same time
                 if 'n_components' in kwargs:
@@ -251,7 +262,11 @@ def wrap_scikits_classifier(scikits_class):
 
         @staticmethod
         def is_trainable():
-            """Return True if the node can be trained, False otherwise."""
+            """Return True if the node can be trained, False otherwise.
+
+            :return: A boolean indicating whether the node can be trained.
+            :rtype: bool
+            """
             return hasattr(scikits_class, 'fit')
 
         # NOTE: at this point scikits nodes can only support up to
@@ -259,7 +274,11 @@ def wrap_scikits_classifier(scikits_class):
         # some reason does not support higher precisions
         def _get_supported_dtypes(self):
             """Return the list of dtypes supported by this node.
-            The types can be specified in any format allowed by numpy.dtype."""
+            The types can be specified in any format allowed by numpy.dtype.
+
+            :return: The list of dtypes supported by this node.
+            :rtype: list
+            """
             return ['float32', 'float64']
 
     # modify class name and docstring
@@ -294,16 +313,32 @@ def wrap_scikits_classifier(scikits_class):
 
 def wrap_scikits_transformer(scikits_class):
     """Wrap a sklearn transformer as an MDP Node subclass.
-
     The wrapper maps these MDP methods to their sklearn equivalents:
-    _stop_training -> fit
-    _execute -> transform
+
+    - _stop_training -> fit
+    - _execute -> transform
+
     """
 
     # create a wrapper class for a sklearn transformer
     class ScikitsNode(mdp.Cumulator):
 
         def __init__(self, input_dim=None, output_dim=None, dtype=None, **kwargs):
+            """
+            Initializes an object of type 'ScikitsNode'.
+
+            :param input_dim: Dimensionality of the input.
+                Default is None.
+            :type input_dim: int
+        
+            :param output_dim: Dimensionality of the output.
+                Default is None.
+            :type output_dim: int
+        
+            :param dtype: Datatype of the input.
+                Default is None.
+            :type dtype: numpy.dtype or str
+            """
             if output_dim is not None:
                 raise ScikitsException(_OUTPUTDIM_ERROR)
             super(ScikitsNode, self).__init__(input_dim=input_dim,
@@ -328,7 +363,11 @@ def wrap_scikits_transformer(scikits_class):
 
         @staticmethod
         def is_trainable():
-            """Return True if the node can be trained, False otherwise."""
+            """Return True if the node can be trained, False otherwise.
+
+            :return: A boolean indication whether the node can be trained.
+            :rtype: bool
+            """
             return hasattr(scikits_class, 'fit')
 
         # NOTE: at this point scikits nodes can only support up to
@@ -336,7 +375,11 @@ def wrap_scikits_transformer(scikits_class):
         # some reason does not support higher precisions
         def _get_supported_dtypes(self):
             """Return the list of dtypes supported by this node.
-            The types can be specified in any format allowed by numpy.dtype."""
+            The types can be specified in any format allowed by numpy.dtype.
+
+            :return: The list of dtypes supported by this node.
+            :rtype: list
+            """
             return ['float32', 'float64']
 
     # modify class name and docstring
@@ -369,16 +412,32 @@ def wrap_scikits_transformer(scikits_class):
 
 def wrap_scikits_predictor(scikits_class):
     """Wrap a sklearn transformer as an MDP Node subclass.
-
     The wrapper maps these MDP methods to their sklearn equivalents:
-    _stop_training -> fit
-    _execute -> predict
+
+    - _stop_training -> fit
+    - _execute -> predict
+    
     """
 
     # create a wrapper class for a sklearn predictor
     class ScikitsNode(mdp.Cumulator):
 
         def __init__(self, input_dim=None, output_dim=None, dtype=None, **kwargs):
+            """
+            Initializes an object of type 'ScikitsNode'.
+
+            :param input_dim: Dimensionality of the input.
+                Default is None.
+            :type input_dim: int
+        
+            :param output_dim: Dimensionality of the output.
+                Default is None.
+            :type output_dim: int
+        
+            :param dtype: Datatype of the input.
+                Default is None.
+            :type dtype: numpy.dtype or str
+            """
             if output_dim is not None:
                 raise ScikitsException(_OUTPUTDIM_ERROR)
             super(ScikitsNode, self).__init__(input_dim=input_dim,
@@ -403,7 +462,10 @@ def wrap_scikits_predictor(scikits_class):
 
         @staticmethod
         def is_trainable():
-            """Return True if the node can be trained, False otherwise."""
+            """Return True if the node can be trained, False otherwise.
+
+            :return: A boolean indicating whether the node can be trained.
+            :rtype: bool"""
             return hasattr(scikits_class, 'fit')
 
         # NOTE: at this point scikits nodes can only support up to 64-bits floats
@@ -484,4 +546,4 @@ apply_to_scikits_algorithms(sklearn,
 DICT_ = {}
 for wrapped_c in scikits_nodes:
     #print wrapped_c.__name__
-    DICT_[wrapped_c.__name__] = wrapped_c
+    DICT_[wrapped_c.__name__] = wrapped_c
\ No newline at end of file
