@@ -1,12 +1,15 @@
 """
 Thread based scheduler for distribution across multiple CPU cores.
 """
+from __future__ import print_function
+from future import standard_library
+standard_library.install_aliases()
 
 import threading
 import time
-import cPickle as pickle
+import pickle as pickle
 
-from scheduling import Scheduler, cpu_count
+from .scheduling import Scheduler, cpu_count
 
 SLEEP_TIME = 0.1  # time spend sleeping when waiting for a thread to finish
 
