@@ -1,7 +1,6 @@
-from __future__ import with_statement
 import mdp.hinet as mh
-from _tools import *
-from test_nodes_generic import (
+from ._tools import *
+from .test_nodes_generic import (
     generic_test_factory,
     test_dtype_consistency,
     # this test fails due to the checks in _set_output_dim
