@@ -1,12 +1,23 @@
 
 import mdp
+# python 2/3 compatibility
+try:
+    from inspect import getfullargspec as getargs
+except ImportError:
+    from inspect import getargspec as getargs
 import inspect
-X = mdp.numx_rand.random(size=(500,5))
+from mdp.utils import inspect_formatargspec
+
+
+X = mdp.numx_rand.random(size=(500, 5))
+
 
 def get_signature(func):
-    regargs, varargs, varkwargs, defaults = inspect.getargspec(func)
-    return inspect.formatargspec(regargs, varargs, varkwargs, defaults,
-                                 formatvalue=lambda value: "")[1:-1]
+    regargs, varargs, varkwargs, defaults = getargs(func)[:4]
+    return inspect_formatargspec(
+            regargs, varargs, varkwargs, defaults,
+            formatvalue=lambda value: "")[1:-1]
+
 
 def test_docstrings():
     # first try on a subclass of Node if
@@ -21,8 +32,8 @@ def test_docstrings():
     assert anode.foo == 42
     assert get_signature(anode.train) == 'self, x'
 
-
     # now try on a subclass of it
+
     class ChildNode(AncestorNode):
         def _train(self, x):
             """doc child"""
@@ -33,6 +44,7 @@ def test_docstrings():
     assert cnode.foo2 == 42
     assert get_signature(cnode.train) == 'self, x'
 
+
 def test_signatures_no_doc():
     # first try on a subclass of Node if
     # the signature is exported to the public method
@@ -43,7 +55,7 @@ def test_signatures_no_doc():
     anode.train(X, foo='abc')
     assert anode.foo == 42
     assert get_signature(anode.train) == 'self, x, foo'
-    
+
     # now try on a subclass of it
     class ChildNode(AncestorNode):
         def _train(self, x, foo2=None):
@@ -53,6 +65,7 @@ def test_signatures_no_doc():
     assert cnode.foo2 == 42
     assert get_signature(cnode.train) == 'self, x, foo2'
 
+
 def test_signatures_with_doc_in_both():
     # first try on a subclass of Node if
     # the signature and the docstring are exported to
@@ -66,7 +79,7 @@ def test_signatures_with_doc_in_both():
     anode.train(X, foo='abc')
     assert anode.foo == 42
     assert get_signature(anode.train) == 'self, x, foo'
-    
+
     # now try on a subclass of it
     class ChildNode(AncestorNode):
         def _train(self, x, foo2=None):
@@ -78,6 +91,7 @@ def test_signatures_with_doc_in_both():
     assert cnode.foo2 == 42
     assert get_signature(cnode.train) == 'self, x, foo2'
 
+
 def test_signatures_with_doc_in_ancestor():
     # first try on a subclass of Node if
     # the signature and the docstring are exported to
@@ -102,6 +116,7 @@ def test_signatures_with_doc_in_ancestor():
     assert cnode.foo2 == 42
     assert get_signature(cnode.train) == 'self, x, foo2'
 
+
 def test_signatures_with_doc_in_child():
     # first try on a subclass of Node if
     # the signature and the docstring are exported to
