@@ -6,16 +6,17 @@ TESTDECIMALS = {numx.dtype('d'): 12,
                 numx.dtype('F'): 3,
                 }
 
+
 def test_eigenproblem(dtype, range, func):
     """Solve a standard eigenvalue problem."""
     dtype = numx.dtype(dtype)
     dim = 5
     if range:
-        range = (2, dim -1)
+        range = (2, dim - 1)
     else:
         range = None
     a = utils.symrand(dim, dtype)+numx.diag([2.1]*dim).astype(dtype)
-    w,z = func(a, range=range)
+    w, z = func(a, range=range)
     # assertions
     assert_type_equal(z.dtype, dtype)
     w = w.astype(dtype)
@@ -23,17 +24,18 @@ def test_eigenproblem(dtype, range, func):
                                     utils.mult(a, z))).real
     assert_array_almost_equal(diag, w, TESTDECIMALS[dtype])
 
+
 def test_geneigenproblem(dtype, range, func):
     """Solve a generalized eigenvalue problem."""
     dtype = numx.dtype(dtype)
     dim = 5
     if range:
-        range = (2, dim -1)
+        range = (2, dim - 1)
     else:
         range = None
     a = utils.symrand(dim, dtype)
     b = utils.symrand(dim, dtype)+numx.diag([2.1]*dim).astype(dtype)
-    w,z = func(a,b,range=range)
+    w, z = func(a, b, range=range)
     # assertions
     assert z.dtype == dtype
     w = w.astype(dtype)
@@ -45,18 +47,23 @@ def test_geneigenproblem(dtype, range, func):
     assert_array_almost_equal(diag2, numx.ones(diag2.shape[0]),
                               TESTDECIMALS[dtype])
 
+
 test_geneigenproblem.funcs = [utils._symeig._symeig_fake]
 if mdp.utils.symeig is utils._symeig.wrap_eigh:
     test_geneigenproblem.funcs.append(utils._symeig.wrap_eigh)
 
 test_eigenproblem.funcs = test_geneigenproblem.funcs + [utils.nongeneral_svd]
 
+
 def pytest_generate_tests(metafunc):
+    ids = []
+    argvalues = []
     for testtype in ('d', 'f'):
         for therange in (False, True):
             for func in metafunc.function.funcs:
-                funcargs = dict(dtype=testtype,
-                                range=therange,
-                                func=func)
-                theid = "%s, %s, %s" % (func.__name__, testtype, therange)
-                metafunc.addcall(funcargs, id=theid)
+                # build lists
+                argvalues.append((testtype, therange, func))
+                ids.append("%s, %s, %s" % (func.__name__, testtype, therange))
+
+    metafunc.parametrize(argnames='dtype,range,func',
+                         argvalues=argvalues, ids=ids)
