@@ -12,7 +12,7 @@ http://javascript.internet.com/miscellaneous/image-slideshow.html
 (which in turn seems to be based on something from http://www.ricocheting.com)
 """
 
-from __future__ import with_statement
+from builtins import str
 
 import random
 import tempfile
@@ -20,7 +20,7 @@ import os
 import webbrowser
 import warnings
 
-import templet
+from . import templet
 
 _BASIC_CSS_FILENAME = "basic.css"
 _SLIDESHOW_CSS_FILENAME = "slideshow.css"
